package geniusweb.voting.evaluatorwithvalue;

import java.util.Collections;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnore;

import geniusweb.inform.Agreements;
import geniusweb.voting.CollectedVotesWithValue;
import geniusweb.voting.VotingEvaluatorWithValue;

/**
 * Take the first possible agreement, and take the largest possible agreement.
 * Stop after first agreement.
 */
public class LargestAgreementWithValue implements VotingEvaluatorWithValue {
	@JsonIgnore
	private final CollectedVotesWithValue allVotes;

	// caching
	@JsonIgnore
	private Agreements maxAgreements = null;

	@JsonCreator
	public LargestAgreementWithValue() {
		this(new CollectedVotesWithValue(Collections.emptyMap(),
				Collections.emptyMap()));
	}

	protected LargestAgreementWithValue(CollectedVotesWithValue votes) {
		this.allVotes = votes;
	}

	@Override
	public Agreements getAgreements() {
		if (maxAgreements == null)
			maxAgreements = new Agreements(allVotes.getMaxAgreements());
		return maxAgreements;
	}

	@Override
	public boolean isFinished() {
		return getAgreements().getMap().isEmpty();
	}

	@Override
	public VotingEvaluatorWithValue create(CollectedVotesWithValue votes) {
		return new LargestAgreementWithValue(votes);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((allVotes == null) ? 0 : allVotes.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		LargestAgreementWithValue other = (LargestAgreementWithValue) obj;
		if (allVotes == null) {
			if (other.allVotes != null)
				return false;
		} else if (!allVotes.equals(other.allVotes))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "LargestAgreement";
	}
}
